package cordova.plugin.encryption;

import android.content.Context;
import android.content.res.AssetManager;

import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Comparator;
import java.util.Map;
import java.util.TreeMap;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * This class echoes a string called from JavaScript.
 */
public class EncryptionPlugin extends CordovaPlugin {

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        if (action.equals("getFileList")) {
            this.getFileList(callbackContext);
            return true;
        }
        if (action.equals("deleteFileList")) {
            this.deleteFileList(callbackContext);
            return true;
        }
        return false;
    }

    //生成fileList.json文件
    private void getFileList(/*String message, */CallbackContext callbackContext) throws JSONException {

        JSONObject jsonObject = new JSONObject();

        try {
            File file = new File(this.webView.getContext().
                    getFilesDir().getAbsolutePath()+"/fileList.json");

            //判断fileList.json文件是否存在
            if(file.exists()) {
                jsonObject.put("fileContent","");
                jsonObject.put("fileIsExists","true");
            } else {
                //创建fileList.json文件
                FileOutputStream fileOutputStream = this.webView.getContext().
                        openFileOutput("fileList.json", Context.MODE_PRIVATE);
                OutputStreamWriter outputStreamWriter = new OutputStreamWriter(fileOutputStream, "UTF-8");

                //加密处理
                AssetManager assetManager = this.webView.getContext().getAssets();
                Map<String, String> map = sortMapByKey(getDirMD5(assetManager, "www"));

                //构造json数据
                outputStreamWriter.write("{");
                int i = 0;
                for (String key :
                        map.keySet()) {
                    i++;
                    if (i < map.size()) {
                        outputStreamWriter.write("\"" + key.substring(4) + "\"" + ":" + "\"" + map.get(key) + "\"" + ",");
                    } else {
                        outputStreamWriter.write("\"" + key.substring(4) + "\"" + ":" + "\"" + map.get(key) + "\"");
                    }
                }
                outputStreamWriter.write("}");

                //关闭资源
                outputStreamWriter.flush();
                fileOutputStream.flush();
                outputStreamWriter.close();
                fileOutputStream.close();

                //读取fileList.json文件
                FileInputStream fis = this.webView.getContext().openFileInput("fileList.json");
                InputStreamReader is = new InputStreamReader(fis, "UTF-8");
                //fis.available()文件可用长度
                char input[] = new char[fis.available()];
                is.read(input);
                is.close();
                fis.close();
                String readed = new String(input);

                jsonObject.put("fileContent",readed);
                jsonObject.put("fileIsExists","false");
            }
            callbackContext.success(jsonObject);
        } catch (UnsupportedEncodingException e) {
            callbackContext.error("error");
        } catch (FileNotFoundException e) {
            callbackContext.error("error");
        } catch (IOException e) {
            callbackContext.error("error");
        }

    }

    //删除fileList.json文件
    private void deleteFileList(CallbackContext callbackContext) {

        //删除fileList.json文件
        boolean flag = this.webView.getContext().deleteFile("fileList.json");

        if (flag) {
            callbackContext.success("success");
        } else {
            callbackContext.error("error");
        }

    }

    /**
     * 根据文件计算出文件的MD5
     * @param inputStream
     * @return string
     */
    private static String getFileMD5(InputStream inputStream) {
        MessageDigest digest = null;
        byte buffer[] = new byte[1024];
        int len;
        try {
            digest = MessageDigest.getInstance("MD5");
            while ((len = inputStream.read(buffer, 0, 1024)) != -1) {
                digest.update(buffer, 0, len);
            }
            inputStream.close();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        BigInteger bigInt = new BigInteger(1, digest.digest());

        return bigInt.toString(16);
    }

    /**
     * 获取文件夹中的文件的MD5值
     * @param path
     * @return map
     */
    private static Map<String,String> getDirMD5(AssetManager assetManager, String path) throws IOException {
        String[] fileNames= assetManager.list(path);
        String md5 = null;
        Map<String, String> map = new TreeMap<String, String>();
        if (fileNames.length > 0) {//如果是文件则长度为0
            for (String fileName:
                    fileNames) {
                map.putAll(getDirMD5(assetManager,path + "/" + fileName));
            }
        } else {
            String fileName = path;
            InputStream inputStream = assetManager.open(fileName);
            md5 = getFileMD5(inputStream);
            if (md5 != null) {
                map.put(fileName,md5);
            }
            return map;
        }
        return map;
    }

    /**
     * 使用 Map按key进行排序
     * @param map
     * @return
     */
    private static Map<String, String> sortMapByKey(Map<String, String> map) {
        if (map == null || map.isEmpty()) {
            return null;
        }

        Map<String, String> sortMap = new TreeMap<String, String>(
                new Comparator<String>(){
                    @Override
                    public int compare(String str1, String str2) {
                        return str1.compareTo(str2);
                    }
                });
        sortMap.putAll(map);
        return sortMap;
    }
}
